package com.diagbot.web;


import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.facade.ConceptInfoFacade;
import com.diagbot.vo.StaticKnowledgeHISVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.diagbot.vo.StaticKnowledgeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 静态知识标准术语表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2020-08-18
 */
@RestController
@RequestMapping("/graph/conceptInfo")
@Api(value = "静态知识标准术语相关API", tags = { "静态知识标准术语相关API" })
public class ConceptInfoController {

    @Autowired
    private ConceptInfoFacade conceptInfoFacade;

    @ApiOperation(value = "医学知识（静态知识）检索[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticKnowledgeIndex")
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> data = conceptInfoFacade.staticKnowledgeIndex(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "页面获取静态知识[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验明细、5-检查、6-手术和操作 <br>" +
                    "contentTypes： 内容类型（多选）：1-静态信息、2-注意事项、3-临床路径 <br>" +
                    "name： 标准术语名称<br>")
    @PostMapping("/getStaticKnowledge")
    public RespDTO<StaticKnowledgeDTO> getStaticKnowledge(@Valid @RequestBody StaticKnowledgeVO staticKnowledgeVO) {
        StaticKnowledgeDTO data = conceptInfoFacade.getStaticKnowledge(staticKnowledgeVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "对接获取静态知识[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验明细、5-检查、6-手术和操作 <br>" +
                    "contentTypes： 内容类型（多选）：1-静态信息、2-注意事项、3-临床路径 <br>" +
                    "hisName： HIS大项名称<br>" +
                    "hisDetailName： HIS小项名称<br>")
    @PostMapping("/getStaticKnowledgeForHIS")
    public RespDTO<List<StaticKnowledgeDTO>> getStaticKnowledgeForHIS(@Valid @RequestBody StaticKnowledgeHISVO staticKnowledgeHISVO) {
        List<StaticKnowledgeDTO> data = conceptInfoFacade.getStaticKnowledgeForHIS(staticKnowledgeHISVO);
        return RespDTO.onSuc(data);
    }
}